//To give names to each node of the tree
const Data = {
  name: 'Asia',
  value: 9,
  type: 'blue',
  level: 'aqua',
  children: [
    {
      name: 'EasternMediterranean',
      value: 9,
      type: 'red',
      level: '#863a9f',
      children: [
        //To Name the sub unit of the EasternMediterranean
        {
          name: 'Pakistan',
          value: 9,
          type: 'green',
          level: 'yellow',
          children: [
            //To Name the sub units of Pakistan
            {
              name: 'Total Cases-281863',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Deaths-6035',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries-256058',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Active Cases-19770',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Serious,Critical-809',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          // To Name another sub unit of EasternMediterranean
          name: 'Saudi Arabia',
          value: 9,
          type: 'green',
          level: 'yellow',
          children: [
            {
              //To name the sub units of Saudi Arabia
              name: 'Total Cases-284226',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Deaths-3055',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
    {
      //To Name the second sub unit of Asia
      name: 'South-EastAsia',
      value: 9,
      type: 'red',
      level: '#863a9f',
      children: [
        {
          //Naming the first sub unit of South-EastAsia
          name: 'Bangladesh',
          value: 9,
          type: 'green',
          level: 'yellow',
          children: [
            {
              name: 'Total Cases-249651',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          //To Name the second sub unit of South-EastAsia
          name: 'India',
          value: 9,
          type: 'green',
          level: 'yellow',
          children: [
            {
              name: 'Total Cases-2025409',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
        {
          // To Name the third sub unit of South-EastAsia
          name: 'Indonesia',
          value: 9,
          type: 'green',
          level: 'yellow',
          children: [
            {
              //To name the children of Indonesia
              name: 'Total Cases-118753',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Deaths-5521',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
            {
              name: 'Total Recoveries-75645',
              value: 9,
              type: '#863a9f',
              level: '#3a9f7e',
            },
          ],
        },
      ],
    },
  ],
};

// To define width,height and margin of the tree
const margin = {
    top: 10,
    right: 250,
    bottom: 200,
    left: 200,
  },
  width = 850 - margin.left - margin.right,
  height = 650 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

//To Assign heirarchical structure to classify the data
let nodes = d3.hierarchy(Data, (node) => node.children);

//To Provide data connection for nodes to their respective position in the tree
nodes = tree(nodes);

//To Add an svg element
const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom),
  g = svg
    .append('g') //appends group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

//To Define links between nodes
const link = g
  .selectAll('.link')
  .data(nodes.descendants().slice(1))
  .enter()
  .append('path')
  .attr('class', 'link')
  .style('stroke', (d) => d.data.level)
  .attr('d', (d) => {
    return (
      'M' +
      d.y +
      ',' +
      d.x +
      'C' +
      (d.y + d.parent.y) / 2 +
      ',' +
      d.x +
      ' ' +
      (d.y + d.parent.y) / 2 +
      ',' +
      d.parent.x +
      ' ' +
      d.parent.y +
      ',' +
      d.parent.x
    );
  });

//To declare a group for each node present in the tree
const node = g
  .selectAll('.node')
  .data(nodes.descendants())
  .enter()
  .append('g')
  .attr(
    'class',
    (d) =>
      'node' +
      (d.children ? ' node--internal' : ' node--leaf'),
  )
  .attr(
    'transform',
    (d) => 'translate(' + d.y + ',' + d.x + ')',
  );

//To Define shape of the node
node
  .append('circle')
  .attr('r', (d) => d.data.value)
  .style('stroke', (d) => d.data.type)
  .style('fill', (d) => d.data.level);

//To add a new node to the list
node
  .append('text')
  .attr('dy', '.60em')
  .attr('x', (d) =>
    d.children ? (d.data.value + 5) * -1 : d.data.value + 5,
  )
  .attr('y', (d) =>
    d.children && d.depth !== 0 ? -(d.data.value + 5) : d,
  )
  .style('text-anchor', (d) =>
    d.children ? 'end' : 'start',
  )
  .text((d) => d.data.name);
